;;
;; Copyright (c) 2003, 2004 Tama Communications Corporation
;;
;; This file is part of BOKIN.
;;
;; BOKIN is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; BOKIN is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software Foundation, Inc.
;; 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
;;
;; Common procedure for BOKIN command.

(define-module (bokin command)
:export (
	bkn:get-version
	bkn:start-command
	bkn:show-version
	bkn:update-file
	bkn:generate-output-comment
))

(use-modules ((ice-9 format)
              :select (format)))
(use-modules ((srfi srfi-13)
              :select (string-join)))
(use-modules (bokin config))
(use-modules (bokin error))

(define required-version "1.6.4")	; must be "[0-9]+\.[0-9]+\.[0-9]+"
(define copyright-year "2003, 2004, 2005")

(define (bkn:start-command)
  (version-check)
  (bkn:load-config)
  (if (bkn:get-config-variable 'backtrace 'boolean)
      (debug-enable 'debug 'backtrace)))

;; Check guile version.
;;
(define (version-check)
  (let ((version<? (lambda (n1 n2)
                    (if (< (string->number n1) (string->number n2)) #t #f)))
        (required-array (string-split required-version #\.)))
    (if (or (version<? (major-version) (list-ref required-array 0))
            (version<? (minor-version) (list-ref required-array 1))
            (version<? (micro-version) (list-ref required-array 2)))
        (bkn:error "Guile ~a is required." required-version))))

(define (bkn:get-version)
  (bkn:get-config-variable 'version))

(define (bkn:show-version program-name verbose)
  (let ((port (current-output-port))
        (bokin-version (bkn:get-version)))
    (if verbose
        (begin
          (display program-name port)
          (display " - BOKIN version " port)
          (display bokin-version port)
          (display "\n" port)
          (display (format "Copyright (c) ~a Tama Communications Corporation\n" copyright-year) port)
          (display "BOKIN comes with NO WARRANTY, to the extent permitted by law.\n" port)
          (display "You may redistribute copies of BOKIN under the terms of the\n" port)
          (display "GNU General Public License. For more information about these matters,\n" port)
          (display "see the files named COPYING.\n" port))
        (begin
          (display bokin-version port)
          (display "\n" port))))
  (exit 0))

;; Update file using Make.
;; Previously, generating rule must be defined in the Makefile.
;;
(define (bkn:update-file path verbose)
  (let ((dir (dirname path))
        (file (basename path))
        (make-path (bkn:get-config-variable 'program:make)))
    (if (not make-path)
        (set! make-path "make"))
    (let ((command-line (string-append
                          "cd "
                          dir
                          ";"
                          make-path
                          " "
                          (if verbose "" "-s ")
                          file)))
      (if verbose
          (display (format "Updating '~a'...\n" file)))
      (system command-line))))

;; Write comments on fixed form to output file.
;;
(define (bkn:generate-output-comment program-name comment-char target . option)
  (let ((file-name (if (pair? option) (car option) #f)))
    (string-append 
      (format "~a\n" comment-char)
      (format "~a ~a produced by ~a version ~a\n"
                  comment-char target program-name (bkn:get-version))
      (format "~a\n" comment-char)
      (if file-name
          (format "~a File:      ~a\n" comment-char file-name) "")
      (format "~a Directory: ~a\n" comment-char (getcwd))
      (format "~a Command:   ~a\n" comment-char (string-join (command-line) " "))
      (format "~a\n" comment-char))))
