;;
;; Copyright (c) 2003, 2004, 2005 Tama Communications Corporation
;;
;; This file is part of BOKIN.
;;
;; BOKIN is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; BOKIN is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software Foundation, Inc.
;; 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
;;
;; Date processing for BOKIN.

(define-module (bokin date)
:export (
	bkn:string->date
	bkn:date->string
	DATE-YEAR
	DATE-MONTH
	DATE-DAY
	DATE-SERIAL
))

(use-modules ((ice-9 regex)
              :select (match:substring)))
(use-modules (bokin error))

;;; Parser.

;; Date format:
;;
;;	date: <year><month><day>[/<serial>]
;;	year:  9999
;;	month: 99
;;	day:   99
;;	serial:9999
;;
(define begin-of-line "^")
(define number "[0-9]+")
(define year "[0-9][0-9][0-9][0-9]")
(define month "[0-9][0-9]")
(define day "[0-9][0-9]")
(define (extract arg)
  (string-append "(" arg ")"))
(define (option arg)
  (string-append "(" arg ")?"))
(define pat-date (string-append
        begin-of-line
        (extract year)                                  ; year
        (extract month)                                 ; month
        (extract day)                                   ; day
        (option
            (string-append "/" (extract number))        ; serial
        )))
(define regex-date (make-regexp pat-date))

;; Convert a date string into the date list.
;;
;; 20050102 => '(2005 1 2)
;; 20050102/3 => '(2005 1 2 3)
;;
(define (bkn:string->date string)
  (let ((match (regexp-exec regex-date string))
        (result #f))

    ; Date string:
    ;
    ; $0: 20031231/1
    ; $1: 2004 -> year
    ; $2: 12   -> month
    ; $3: 31   -> day
    ; $4: -1
    ; $5: 1    -> serial(optional)
    ;
    (if match
        (begin
          (set! result (list (string->number (match:substring match 1))
                             (string->number (match:substring match 2))
                             (string->number (match:substring match 3))))
          (if (match:substring match 5)
              (append! result (list (string->number (match:substring match 5)))))))
    result))

;;; Date object

(define DATE-YEAR 0)
(define DATE-MONTH 1)
(define DATE-DAY 2)
(define DATE-SERIAL 3)

(define (number->02string n)
  (string-append (if (< n 10) "0" "") (number->string n)))

;; Convert a date list or vector into the date string.
;;
;; This procedure accept both list and vector.
;;
;; '(2005 1 2) => 20050102
;; '(2005 1 2 3) => 20050102/3
;;
(define (bkn:date->string date)
  (let ((any-ref (if (vector? date) vector-ref list-ref))
        (any-length (if (vector? date) vector-length length)))
    (string-append (number->02string (any-ref date DATE-YEAR))
                   (number->02string (any-ref date DATE-MONTH))
                   (number->02string (any-ref date DATE-DAY))
                   (if (and (= (any-length date) 4) (> (any-ref date DATE-SERIAL) 0))
                       (string-append "/" (number->string (any-ref date DATE-SERIAL)))
                       ""))))
