;;
;; Copyright (c) 2004 Tama Communications Corporation
;;
;; This file is part of BOKIN.
;;
;; BOKIN is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; BOKIN is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software Foundation, Inc.
;; 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
;;
;; Utility for directory.

(define-module (bokin directory)
:export (
        bkn:make-file-list
	bkn:get-period-directory-list
	bkn:directory?
))

(use-modules ((ice-9 rdelim)
              :select (read-line)))
(use-modules ((srfi srfi-13)
              :select (string-join)))

;; Make file list.
;;
;; Lookup specified file from the current directory to the PROJECT_DIRECTORY.
;; return (<PROJECT_DIRECTORY> file-list ...)
;;
(define (bkn:make-file-list file . option)
  (let ((verbose (if (pair? option) (car option) #f))
        (filelist '())
        (topdir #f))

    ; Construct list of configuration file.
    (let loop ((dirlist (reverse (string-split (getcwd) #\/))))
       (if (not (null? dirlist))
           (let* ((dir (string-join (reverse dirlist) "/"))
                  (path (string-append dir "/" file))
                  (top (string-append dir "/" "PROJECT_DIRECTORY")))
             (if (access? path R_OK)
                 (set! filelist (cons path filelist)))
             (if (access? top R_OK)
                 (set! topdir dir)      ; Break loop because PROJECT_DIRECTORY is found.
                 (loop (cdr dirlist))))))

    ; Insert PROJECT_DIRECTORY at the head of the filelist.
    (set! filelist (cons topdir filelist))
    filelist))

(define (get-list-from-file file)
  (let ((file-list '()))
    (call-with-input-file file
      (lambda (port)
        (do ((line (read-line port) (read-line port)))
            ((eof-object? line) file-list)
          (set! file-list (cons line file-list)))))))

;; Get period directory list.
;;
;; (<curdir> <period-directories>)
;;
(define (bkn:get-period-directory-list)
  (let ((curdir '())
        (topdir #f)
        (topfile #f))

    ; Construct list of configuration file.
    (let loop ((dirlist (reverse (string-split (getcwd) #\/))))
       (if (not (null? dirlist))
           (let* ((dir (string-join (reverse dirlist) "/")))
             (set! topfile (string-append dir "/" "PROJECT_DIRECTORY"))
             (if (access? topfile R_OK)
                 (set! topdir dir)
                 (begin
                   (set! curdir (cons (car dirlist) curdir))
                   (loop (cdr dirlist)))))))
    (cons (string-join curdir "/") (reverse (get-list-from-file topfile)))))

;; directory?
;;
(define (bkn:directory? name)
  (if (and (access? name (logand R_OK X_OK))
           (let ((p (stat name)))
             (eq? (stat:type p) 'directory)))
      #t #f))
