;;
;; Copyright (c) 2003, 2004 Tama Communications Corporation
;;
;; This file is part of BOKIN.
;;
;; BOKIN is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; BOKIN is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software Foundation, Inc.
;; 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
;;
;; error processing for BOKIN

(define-module (bokin error)
:export (
	bkn:set-current-line
	bkn:unset-current-line
	bkn:error
	bkn:error-with-line
	bkn:warning
))

(use-modules ((ice-9 format)
              :select (format)))
(use-modules (bokin command))
(use-modules (bokin config))

(define eport (current-error-port))

(define current-line #f)

(define (bkn:set-current-line line)
  (set! current-line line))

(define (bkn:unset-current-line)
  (set! current-line #f))

(define (bkn:warning format-string . args)
  (let ((program-name (basename (car (command-line)))))
    (display (string-append (if program-name
                                (string-append program-name ": ")
                                "")
                            (apply format #f format-string args)
                            "\n") eport)
    (if current-line
        (display (string-append "> " current-line "\n") eport))))

(define (bkn:error-status status format-string . args)
  (apply bkn:warning format-string args)
  (exit status))

(define (bkn:error format-string . args)
  (let ((verbose (bkn:get-config-variable 'backtrace 'boolean)))
    (apply bkn:error-status 1 format-string args)
    (if verbose
        (backtrace))))
