;;
;; Copyright (c) 2003, 2004 Tama Communications Corporation
;;
;; This file is part of BOKIN.
;;
;; BOKIN is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; BOKIN is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software Foundation, Inc.
;; 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
;;
;; Money processing for BOKIN

(define-module (bokin money)
:export (
	bkn:set-comma-interval!
	bkn:set-negative-format-proc
	bkn:money-format
))

(use-modules (bokin config))
(use-modules (bokin error))

;;; Money.

;; Interval of comma.
;;
(define comma-interval 3)

;; Negative format procedure.
;;
(define negative-format (lambda (s) (string-append "-" s)))

(define (bkn:set-negative-format-proc proc)
  (set! negative-format proc))

;; Set comma interval.
;;
(define (bkn:set-comma-interval! n)
  (set! comma-interval n))

;; Insert commas.
;;
(define (insert-comma n)
  (let* ((result '())
         (major (inexact->exact (truncate n)))
         (minor (- n major))
         (char-zero (char->integer #\0)))
    (do ((i 0 (+ i 1))
         (remain (abs major) (quotient remain 10)))
        ((= remain 0)
         (string-append (if (null? result)
                            "0"
                            (list->string result))
                        (if (> minor 0)
                            (substring (number->string minor) 1)
                            "")))
      (if (and (> i 0) (= (remainder i comma-interval) 0))
          (set! result (cons #\, result)))
      (set! result (cons (integer->char (+ char-zero (remainder remain 10)))
                         result)))))

;; Convert number into money format.
;;
(define (bkn:money-format n)
  (if (< n 0)
      (negative-format (insert-comma (abs n)))
      (insert-comma n)))
